structure Gameserver :> GAMESERVER =
struct
	(* A lehetsges legkisebb s legnagyobb gondolt szm *)
	val min = 0
	val max = 100

	(* Jtk vgt jelz kivtel *)
	exception Finished

	(* play secret = (ticket, winner), ahol:
		- winner annak a kliensnek a neve, aki elszr kitallja a secret szmot;
		- ticket egy URL, amin t a kliensek elrhetik a szerver tallgat fggvnyeit.

	   Ha secret nem a [min,max] intervallumbl val, Domain kivtelt eredmnyez.
	 *)
	fun	play secret =
	let
		(* A nyertes neve *)
		val winner : string Promise.promise = Promise.promise ()
		(* A nyertess avatsok szmnak korltozsra szolgl vltoz *)
		val semaphor = ref 1
		(* win name = true, s a nyertes neve name lesz.

		   Ha mr van nyertes, Finished kivtelt eredmnyez
		 *)
		fun win name =
		let
			val s = Ref.exchange (semaphor, 0)
		in
			if s = 0 then raise Finished else Promise.fulfill (winner, name); true
		end
		(* A kliensek tallgat fggvnyei *)
		fun ask'   f         = if ! semaphor = 0 then raise Finished else f secret
		fun solve' (x, name) = if ! semaphor = 0 then raise Finished else if x = secret then win name else false
		(* A kliensek fggvnyeibl kpzett struktra *)
		structure Connection =
			struct
				val ask   = Remote.proxy ask'
				val solve = Remote.proxy solve'
				exception Finished = Finished
			end
		val t = Remote.offer (pack Connection :> GAMECONNECTION)
	in
		if (secret<min orelse secret>max) then
			raise Domain
		else
			(t, Promise.future winner)
	end
end
